PROGRAM_NAME='IP Communications Template'
(***********************************************************)
(*  FILE_LAST_MODIFIED_ON: 01/29/2019  AT: 09:48:47        *)
(***********************************************************)
#include 'SNAPI'
#include 'G4API'
//#include 'Globals'

DEFINE_CONSTANT
//!!! Defne in main.axs !!!
 dev dvIP_IPCOMM = 0:3:0;
 long TL_IPCOMM_CONNECT = 5001; 

DEFINE_TYPE

// !! This could go in gloabls.axi - universal component !!!
structure _IpConnect
{
    char isConnected;
    char address[255];	// Support max fqdns
    integer port;
    char mode;		// TCP or UDP
}

DEFINE_VARIABLE

// !! This could go in globals.axi - universal component !!!
volatile long lTlIpConnect[ ] = {30000}; // 30-second connection check

volatile _IpConnect uIpComm;
volatile char sIpCommBuffer[5000];
volatile char bIpCommDebug;


// !!! put in globals.axi - universal application !!!
define_function char[100] GetIpError (long iErrorCode) 
{
    char iReturn[100]
	
    switch(iErrorCode) 
    {
	case  1 : iReturn = "'Invalid server address'";
	case  2 : iReturn = "'Invalid server port'";
	case  3 : iReturn = "'Invalid value for Protocol'";
	case  4 : iReturn = "'Unable to open communication port with server'";
	case  6 : iReturn = "'Connection refused'";
	case  7 : iReturn = "'Connection timed out'";
	case  8 : iReturn = "'Unknown connection error'";
	case  9 : iReturn = "'Already closed'";
	case 10 : iReturn = "'Binding error'";
	case 11 : iReturn = "'Listening error'";
	case 14 : iReturn = "'Local port already used'";
	case 15 : iReturn = "'UDP socket already listening'";
	case 16 : iReturn = "'Too many open sockets'";
	default : iReturn = "'(',itoa(iErrorCode),')Undefined'";
    }
    return iReturn
}



DEFINE_START

// Data init - set in main.axs for include file re-use
uIpComm.address = '192.168.2.202';
uIpComm.port = 201;
uIpComm.mode = IP_TCP;

create_buffer dvIP_IPCOMM, sIpCommBuffer;

timeline_create(TL_IPCOMM_CONNECT, lTlIpConnect, length_array(lTlIpConnect), TIMELINE_ABSOLUTE, TIMELINE_REPEAT);


DEFINE_EVENT

timeline_event[TL_IPCOMM_CONNECT]
{
    if(uIpComm.isConnected == FALSE)
    {
	if(length_string(uIpComm.address) && uIpComm.port)	// Validate the data
	{
	    ip_client_open(dvIP_IPCOMM.PORT, uIpComm.address, uIpComm.port, uIpComm.mode)
	}
	else
	{
	    amx_log(AMX_ERROR, 'ERROR-> IP Addressing not set for dvIP_IPCOMM');
	}
    }   
}

data_event[dvIP_IPCOMM]
{
    online:
    {
	uIpComm.isConnected = TRUE;
    }
    offline:
    {
	uIpComm.isConnected = FALSE;
 	if(timeline_active(TL_IPCOMM_CONNECT))
	{
	    timeline_set(TL_IPCOMM_CONNECT, lTlIpConnect[1] - 100);	// Minimize offline time
	}
	else 
	{
	    timeline_create(TL_IPCOMM_CONNECT, lTlIpConnect, length_array(lTlIpConnect), TIMELINE_ABSOLUTE, TIMELINE_REPEAT);
	}
    }
    onerror:
    {
	amx_log(AMX_ERROR,"'data_event[dvIP_IPCOMM]{onerror: ',GetIpError(data.number)");
 	if(timeline_active(TL_IPCOMM_CONNECT))
	{
	    switch (data.number)
	    {
		case 7:	// Timed out
		{
		    timeline_set(TL_IPCOMM_CONNECT, lTlIpConnect[1] - 100);	// Minimize offline time
		}
		default:
		{
		    timeline_set(TL_IPCOMM_CONNECT, 0);			// Give it a chance to recover
		}
	    }
	}
	else 
	{
	    timeline_create(TL_IPCOMM_CONNECT, lTlIpConnect, length_array(lTlIpConnect), TIMELINE_ABSOLUTE, TIMELINE_REPEAT);
	}
    }
    string:
    {
	if(bIpCommDebug == TRUE)
	{
	    send_string 0,"'dvIP_IPCOMM:string = ', data.text";
	}
	// Parse complete messages out of buffer - set terminator as per device API
	while(find_string(sIpCommBuffer, "$0D", 1))
	{
	    stack_var char iNibble[255];
	    
	    iNibble = remove_string(sIpCommBuffer, "$0D", 1)
	    // Parse iNibble for relevant data
	}
    }
}